/*
 *  Timer.h
 */

#ifndef Timer_h
#define Timer_h 1

#if defined(__APPLE__) || defined(MACOSX)
	#include <sys/time.h>
#elif defined(WIN32)
	#include "timeval.h"
#endif

#include "BasicTypes.h"

class tTimer
{
public:

	tTimer(); 
	~tTimer(); 
	
	void	mInitializeTimer(); 
	
	inline UInt32	mGetMilliseconds(); 
	inline UInt32	mGetSeconds(); 
	
	inline void		mUpdateTimer(); 
	inline UInt32	mGetMillisecondsSinceLastUpdate(); 
	inline double	mGetSecondsSinceLastUpdate(); 
	
	inline UInt32	mSecondsElapsedSinceLastUpdate(); 
	
protected:

	UInt32	fTimeOfLastUpdate; 
	UInt32	fMillisecondsSinceLastUpdate;

#if defined(__APPLE__) || defined(MACOSX)
	struct timeval	fTimeInit; 
#elif defined(WIN32)
	struct timeval_	fTimeInit; 
#endif
};

inline UInt32 tTimer::mGetMilliseconds()
{
#if defined(__APPLE__) || defined(MACOSX)
	struct timeval cTime; 	
#elif defined(WIN32)
	struct timeval_ cTime; 	
#endif

	gettimeofday(&cTime, 0); 
	return ((cTime.tv_sec - fTimeInit.tv_sec) * 1000) + ((cTime.tv_usec - fTimeInit.tv_usec) / 1000); 
}

inline UInt32 tTimer::mGetSeconds()
{
	return mGetMilliseconds() / 1000; 
}

inline void tTimer::mUpdateTimer()
{
	if ( !fTimeOfLastUpdate )
	{
		fTimeOfLastUpdate = mGetMilliseconds();
	}
	
	fMillisecondsSinceLastUpdate = mGetMilliseconds() - fTimeOfLastUpdate; 
	fTimeOfLastUpdate = mGetMilliseconds();
}

inline UInt32 tTimer::mGetMillisecondsSinceLastUpdate()
{
	return fMillisecondsSinceLastUpdate; 
}

inline double tTimer::mGetSecondsSinceLastUpdate()
{
	return fMillisecondsSinceLastUpdate / 1000.0f; 
}

inline UInt32 tTimer::mSecondsElapsedSinceLastUpdate()
{
	return (mGetMilliseconds() - fTimeOfLastUpdate) / 1000; 
}

extern tTimer theTimer; 

#endif
